@@ -3,7 +3,7 @@ require 'racc/parser'
 #
 # parsemail.rb
 #
-#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+#   Copyright (c) 1998-2001 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -15,10 +15,14 @@ require 'tmail/scanmail'
 
 module TMail
 
+  class Parser < ::Racc::Parser
 
-class Mailp < Racc::Parser
+module_eval <<'..end mailp.y modeval..idbd91c15b7d', 'mailp.y', 290
+
+  def self.parse( str, obj, ident )
+    new.parse( str, obj, ident )
+  end
 
-module_eval <<'..end mailp.y modeval..id597aecd12c', 'mailp.y', 293
 
   MAILP_DEBUG = false
 
@@ -36,14 +40,9 @@ module_eval <<'..end mailp.y modeval..id597aecd12c', 'mailp.y', 293
   end
 
 
-  def Mailp.parse( str, obj, ident )
-    new.parse( str, obj, ident )
-  end
-
-
   NATIVE_ROUTINE = {
     'TMail::MsgidH' => :msgid_parse,
-    'TMail::RefH' => :refs_parse
+    'TMail::RefH'   => :refs_parse
   }
 
   def parse( str, obj, ident )
@@ -52,7 +51,7 @@ module_eval <<'..end mailp.y modeval..id597aecd12c', 'mailp.y', 293
     @field = obj
 
     if mid = NATIVE_ROUTINE[ obj.type.name ] then
-      send mid, str
+      __send__ mid, str
     else
       unless ident then
         ident = obj.type.name.split('::')[-1].to_s
@@ -62,36 +61,28 @@ module_eval <<'..end mailp.y modeval..id597aecd12c', 'mailp.y', 293
         cmt = nil
       end
 
-      @scanner = MailScanner.new( str, ident, cmt )
+      @scanner = Scanner.new( str, ident, cmt )
       @scanner.debug = @scanner_debug
       @first = [ ident.intern, ident ]
-      @pass_array = [nil, nil]
 
-      do_parse
+      yyparse self, :parse_in
     end
   end
 
-
   private
 
 
-  def next_token
-    if @first then
-      ret = @first
-      @first = nil
-      ret
-    else
-      @scanner.scan @pass_array
-    end
+  def parse_in( &block )
+    yield @first
+    @scanner.scan &block
   end
   
   def on_error( tok, val, vstack )
-    raise ParseError,
-      "\nparse error in '#{@field.name}:' on token #{val.inspect}"
+    raise MailSyntaxError,
+          "\nparse error in '#{@field.name}:' on token #{val.inspect}"
   end
 
 
-
   def refs_parse( str )
     arr = []
 
@@ -127,13 +118,13 @@ module_eval <<'..end mailp.y modeval..id597aecd12c', 'mailp.y', 293
     if mdata = ::TMail::MSGID.match( str ) then
       @field.msgid = mdata.to_s
     else
-      raise ParseError, "wrong Message-ID format: #{str}"
+      raise MailSyntaxError, "wrong Message-ID format: #{str}"
     end
   end
 
-..end mailp.y modeval..id597aecd12c
+..end mailp.y modeval..idbd91c15b7d
 
-##### racc 1.3.5 generates ###
+##### racc 1.3.6 generates ###
 
 racc_reduce_table = [
  0, 0, :racc_error,
@@ -951,7 +942,6 @@ module_eval <<'.,.,', 'mailp.y', 261
   result
  end
 
-end   # class Mailp
-
+  end   # class Parser
 
 end   # module TMail
